﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ThinkGeo.MapSuite.Core;
using System.Windows.Media;
using System.ComponentModel;

namespace WpfApplication1
{
    public class LayoutPointStyle : PointStyle
    {

        const int SIZE_SMALL = 8;
        const int SIZE_MEDIUM = 12;
        const int SIZE_LARGE = 14;

        System.Drawing.Bitmap plannedReceiver = null;
        System.Drawing.Bitmap plannedLTU = null;
        System.Drawing.Bitmap plannedSource = null;


        public LayoutPointStyle()
        {
            InitPoints();
        }

        private void InitPoints()
        {

            plannedReceiver = GetCirclePointBitMap(SIZE_SMALL);
            plannedLTU = GetSquarePointBitMap(SIZE_SMALL);
            plannedSource = GetCrossPointBitMap(SIZE_MEDIUM);

        }

        private System.Drawing.Bitmap GetCrossPointBitMap(int bitMapSize)
        {
            System.Drawing.Bitmap bmp = new System.Drawing.Bitmap(bitMapSize, bitMapSize);
            System.Drawing.Graphics gBMP = System.Drawing.Graphics.FromImage(bmp);
            System.Drawing.Brush brush = GetBrush(Colors.Green);
            gBMP.CompositingMode = System.Drawing.Drawing2D.CompositingMode.SourceCopy;
            gBMP.CompositingQuality = System.Drawing.Drawing2D.CompositingQuality.HighQuality;
            int offset = bitMapSize / 3;
            int thickness = bitMapSize - (2 * offset);
            System.Drawing.Rectangle[] rectangles = new System.Drawing.Rectangle[] {
                new System.Drawing.Rectangle(offset, 0, thickness, bitMapSize),
                new System.Drawing.Rectangle(0, offset, bitMapSize, thickness)
            };
            gBMP.FillRectangles(brush, rectangles);
            gBMP.Dispose();
            brush.Dispose();
            return bmp;
        }

        private System.Drawing.Bitmap GetSquarePointBitMap(int bitMapSize)
        {
            System.Drawing.Bitmap bmp = new System.Drawing.Bitmap(bitMapSize, bitMapSize);
            System.Drawing.Graphics gBMP = System.Drawing.Graphics.FromImage(bmp);
            System.Drawing.Brush brush = GetBrush(Colors.Red);
            gBMP.CompositingMode = System.Drawing.Drawing2D.CompositingMode.SourceCopy;
            gBMP.CompositingQuality = System.Drawing.Drawing2D.CompositingQuality.HighQuality;
            gBMP.FillRectangle(brush, 0, 0, bitMapSize, bitMapSize);
            gBMP.Dispose();
            brush.Dispose();

            return bmp;
        }

        private System.Drawing.Bitmap GetCirclePointBitMap(int bitMapSize)
        {

            System.Drawing.Bitmap bmp = new System.Drawing.Bitmap(bitMapSize, bitMapSize);
            System.Drawing.Graphics gBMP = System.Drawing.Graphics.FromImage(bmp);
            System.Drawing.Brush brush = GetBrush(Colors.Blue);
            gBMP.CompositingMode = System.Drawing.Drawing2D.CompositingMode.SourceCopy;
            gBMP.CompositingQuality = System.Drawing.Drawing2D.CompositingQuality.HighQuality;
            gBMP.FillEllipse(brush, 0, 0, bitMapSize - 1, bitMapSize - 1);
            gBMP.Dispose();
            brush.Dispose();
            return bmp;
        }

        private System.Drawing.Brush GetBrush(Color brushColor)
        {
            System.Drawing.Color color = System.Drawing.Color.FromArgb(brushColor.A, brushColor.R, brushColor.G, brushColor.B);
            return new System.Drawing.SolidBrush(color);
        }

        protected override void DrawCore(IEnumerable<Feature> features, GeoCanvas canvas, System.Collections.ObjectModel.Collection<SimpleCandidate> labelsInThisLayer, System.Collections.ObjectModel.Collection<SimpleCandidate> labelsInAllLayers)
        {

            GdiPlusGeoCanvas gcanvas = canvas as GdiPlusGeoCanvas;
            gcanvas.CompositingQuality = System.Drawing.Drawing2D.CompositingQuality.HighSpeed;
            gcanvas.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighSpeed;
            gcanvas.DrawingQuality = DrawingQuality.HighSpeed;

            foreach (Feature f in features)
            {
                if (f.Tag is Point)
                {

                    System.Drawing.Bitmap toDraw = null;
                    Point p = f.Tag as Point;
                    switch (p.DisplayType)
                    {

                        case PointDisplayType.PLANNED_RECEIVER:
                            toDraw = plannedReceiver;
                            break;

                        case PointDisplayType.PLANNED_LTU:
                            toDraw = plannedLTU;
                            break;

                        case PointDisplayType.PLANNED_SOURCE:
                            toDraw = plannedSource;
                            break;

                    }

                    PointShape pointShape = (PointShape)f.GetShape();
                    gcanvas.DrawWorldImageWithoutScaling(toDraw, pointShape.X, pointShape.Y, DrawingLevel.LevelOne);
                    //gcanvas.DrawWorldImageWithoutScaling(toDraw, p.DisplayLon, p.DisplayLat, DrawingLevel.LevelOne);
                }

            }
        }
    }
    public enum PointDisplayType { PLANNED_SOURCE, PLANNED_LTU, PLANNED_RECEIVER };
    public class Point : INotifyPropertyChanged
    {
        private PointDisplayType dispType;

        public PointDisplayType DisplayType
        {
            get { return dispType; }
            set {
                if (dispType != value)
                {
                    dispType = value;
                    NotifyPropertyChanged("DisplayType");
                }
            }
        }
        private double displayLon;

        public double DisplayLon
        {
            get { return displayLon; }
            set {
                if (displayLon != value)
                {
                    displayLon = value;
                    NotifyPropertyChanged("DisplayLon");
                }
            }
        }
        private double displayLat;

        public double DisplayLat
        {
            get { return displayLat; }
            set {
                if (displayLat != value)
                {
                    displayLat = value;
                    NotifyPropertyChanged("DisplayLat");
                }
            }
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;
        protected void NotifyPropertyChanged(String property)
        {
            
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(property));
            }
        }
        #endregion
    }


}
