﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;
using System.Collections.ObjectModel;
using System.Windows.Threading;
using System.Collections.Specialized;

namespace WpfApplication1
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        const string POINTS_OVERLAY_NAME = "PointsOverLay";
        const string POINTS_LAYER_NAME = "PointsLayer";

        private bool updating = false;

        LayerOverlay pointsOverlay = null;
        PointsFeatureLayer pfLayer = null;

        private RotationProjection rotateProjection;  // <------

        private BatchObservableCollection<Point> receiverPoints;
        public BatchObservableCollection<Point> ReceiverPoints
        {
            get
            {
                if (receiverPoints == null)
                    receiverPoints = new BatchObservableCollection<Point>();
                return receiverPoints;
            }
            set { receiverPoints = value; }
        }

        private BatchObservableCollection<Point> shotPoints;
        public BatchObservableCollection<Point> ShotPoints
        {
            get
            {
                if (shotPoints == null)
                    shotPoints = new BatchObservableCollection<Point>();
                return shotPoints;
            }
            set { shotPoints = value; }
        }

        private BatchObservableCollection<Point> ltuPoints;
        public BatchObservableCollection<Point> LTUPoints
        {
            get
            {
                if (ltuPoints == null)
                    ltuPoints = new BatchObservableCollection<Point>();
                return ltuPoints;
            }
            set { ltuPoints = value; }
        }

        public double MapAngle
        {
            get
            {
                double val = 0f;
                double.TryParse(textBoxMapAngle.Text, out val);
                return val;
            }
            set
            {
                textBoxMapAngle.Text = value.ToString();
            }
        }

        public Window1()
        {
            InitializeComponent();
        }

        private void wpfmap_Loaded(object sender, RoutedEventArgs e)
        {
            InitializeData();
            WpfMap map = (WpfMap)sender;
            map.MapUnit = GeographyUnit.DecimalDegree;
            map.CurrentExtent = new RectangleShape(0, 100, 100, 0);
            map.ThreadingMode = MapThreadingMode.SingleThreaded;

            map.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.StandardColors.White);
            rotateProjection = new RotationProjection();
            MapAngle = 20;
            rotateProjection.Angle = MapAngle;


            pointsOverlay = new LayerOverlay();

            pfLayer = new PointsFeatureLayer(this);
            pfLayer.FeatureSource.Projection = rotateProjection;  // <------ un-comment

            pointsOverlay.Layers.Add(POINTS_LAYER_NAME, pfLayer);
            map.Overlays.Add(POINTS_OVERLAY_NAME, pointsOverlay);

            pfLayer.FeatureSource.Open();
            //map.CurrentExtent = pfLayer.FeatureSource.GetBoundingBox();      // <----- comment out
            map.CurrentExtent = rotateProjection.GetUpdatedExtent(pfLayer.FeatureSource.GetBoundingBox());  // <------ un-comemnt
            map.CenterAt(pfLayer.GetBoundingBox().GetCenterPoint());

            map.Refresh();
        }

        private void InitializeData()
        {
            double startLat = 35.59526667;
            double startLon = -99.55963611;
            double latIncr = .00055;
            double lonIncr = .00538;

            double curLon = startLon;
            double curLat = startLat;
            List<Point> pts = new List<Point>();
            for (int i = 0; i <20; i++)
            {
                curLat = startLat;
                for (int j = 0; j < 500; j++)
                {
                    Point p = new Point();
                    p.DisplayLat = curLat;
                    p.DisplayLon = curLon;
                    p.DisplayType = PointDisplayType.PLANNED_RECEIVER;
                    pts.Add(p);
                    curLat += latIncr;
                }
                curLon += lonIncr;
            }
            ReceiverPoints.BatchReplace(pts);

            pts = new List<Point>();
            curLon = startLon + (lonIncr / 2);
            curLat = startLat;
            for (int i = 0; i < 20; i++)
            {
                curLat = startLat;
                for (int j = 0; j < 500; j++)
                {
                    Point p = new Point();
                    p.DisplayLat = curLat;
                    p.DisplayLon = curLon;
                    p.DisplayType = PointDisplayType.PLANNED_SOURCE;
                    pts.Add(p);
                    curLat += latIncr;
                }
                curLon += lonIncr;
            }
            ShotPoints.BatchReplace(pts);

        }

        private delegate void DRefreshView();

        bool pendingRefreshRequest = false;
        private DispatcherPriority pendingPriority = DispatcherPriority.Invalid;

        private delegate void RefreshDelegate();

        public void RefreshView(DispatcherPriority priority)
        {


            bool doRefresh = false;
            lock (this)
            {
                // If a thread has not already started a refresh, then 
                // we want to kick one off
                if (!updating)
                {
                    doRefresh = true;
                    updating = true;
                }
                else
                {
                    // Already a refresh underway on the Dispatcher thread
                    // Just capture the fact that we have a pending request
                    pendingRefreshRequest = true;
                    if (priority > pendingPriority)
                        pendingPriority = priority;

                }
            }

            // If there is no refresh running, then start one up
            if (doRefresh)
            {
                // If we are not on the right thread (i.e. WPF Dispatcher), then queue
                // up a request to the Dispatcher to run the refresh
                if (!Dispatcher.CheckAccess())
                {
                    Delegate refresh = new RefreshDelegate(PerformRefresh);
                    Dispatcher.BeginInvoke(refresh, priority, null);
                }
                else
                {
                    // On the right thread, just do the refresh
                    PerformRefresh();
                }
            }


        }
        /*
         * Call the map geo Refresh method to do the actual refresh
         * This refreshes the points overlay and the line segment overlay
         */
        private void PerformRefresh()
        {

            // Refresh the points overlay
            this.wpfMap1.CurrentExtent = rotateProjection.GetUpdatedExtent(this.wpfMap1.CurrentExtent);
            this.wpfMap1.Refresh(wpfMap1.Overlays[POINTS_OVERLAY_NAME]);

            // See if we have a pending refresh request
            lock (this)
            {
                // Pending refrsh, then go ahead and queue this up again for the dispatcher
                if (pendingRefreshRequest)
                {
                    pendingRefreshRequest = false;
                    RefreshDelegate refresh = new RefreshDelegate(PerformRefresh);
                    DispatcherPriority curPendingPriority = pendingPriority;
                    pendingPriority = DispatcherPriority.Invalid;
                    Dispatcher.BeginInvoke(refresh, curPendingPriority, null);

                }
                else
                    updating = false;
            }
        }

        private void buttonSetMapAngle_Click(object sender, RoutedEventArgs e)
        {
            MapAngle = MapAngle;// +30;
            rotateProjection.Angle += MapAngle;  // <------

            PerformRefresh();
        }

    }
    public class BatchObservableCollection<T> : ObservableCollection<T>
    {
        public BatchObservableCollection()
            : base()
        {

        }

        public BatchObservableCollection(List<T> members)
            : base(members)
        {

        }

        public virtual void BatchReplace(List<T> newItems)
        {
            List<T> oldItems = Items.ToList<T>();

            if (!(newItems.Count == 0 && oldItems.Count == 0))
            {  // If both lists are 0, a range exception is thrown
                Items.Clear();

                foreach (var data in newItems)
                {
                    Items.Add(data);
                }

                NotifyCollectionChangedEventArgs args =
                    new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Replace,
                        newItems, oldItems);
                OnCollectionChanged(args);
            }
        }
    }
}
